==> loadmychunks-1.2.0%2B1.19.4%2Bforge-sources.jar: com/drathonix/loadmychunks/common/mixin/cct/MixinTurtleMoveCommand.java <==
package com.drathonix.loadmychunks.common.mixin.cct;

//? if !computercraft {
/*import com.drathonix.loadmychunks.common.LoadMyChunks;
import org.spongepowered.asm.mixin.Mixin;

@Mixin(LoadMyChunks.class)
public class MixinTurtleMoveCommand {

}
*///?} else {
import com.drathonix.loadmychunks.common.integ.cct.bridge.ITurtleBrainMixin;
import com.drathonix.loadmychunks.common.integ.cct.turtle.TurtleChunkLoaderPeripheral;
import com.drathonix.loadmychunks.common.system.ChunkDataManager;
import dan200.computercraft.api.peripheral.IPeripheral;
import dan200.computercraft.api.turtle.ITurtleAccess;
import dan200.computercraft.api.turtle.TurtleCommandResult;
import dan200.computercraft.api.turtle.TurtleSide;
//? if <=1.19.2 {
/*import dan200.computercraft.shared.TurtlePermissions;
import dan200.computercraft.ComputerCraft;
*///?}
import dan200.computercraft.shared.turtle.core.TurtleBrain;
import dan200.computercraft.shared.turtle.core.TurtleMoveCommand;
import dan200.computercraft.shared.turtle.core.TurtlePlayer;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.Level;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import org.spongepowered.asm.mixin.injection.callback.LocalCapture;

@SuppressWarnings("all")
@Mixin(value = TurtleMoveCommand.class
//? if forge && <=1.20.1
, remap=false
)
public class MixinTurtleMoveCommand {

    //? if <=1.19.2 {
    /*@Redirect(method = "execute",at = @At(value = "INVOKE", target = "Ldan200/computercraft/api/turtle/ITurtleAccess;teleportTo(Lnet/minecraft/world/level/Level;Lnet/minecraft/core/BlockPos;)Z")
            //? if forge && <=1.16.5
            /^,remap = true^/
    )
    public boolean checkCanTP(ITurtleAccess turtle, Level oldWorld, BlockPos newPosition){
        boolean stable = oldWorld.isLoaded(newPosition);
        if(oldWorld instanceof ServerLevel) {
            stable = stable || ((ITurtleBrainMixin)turtle).lmc$preMove((ServerLevel)oldWorld,newPosition);
        }
        if(stable){
            return turtle.teleportTo(oldWorld,newPosition);
        }
        return false;
    }
    *///?} else {
    @Inject(method = "execute",at = @At(value = "INVOKE",target = "Ldan200/computercraft/api/turtle/ITurtleAccess;teleportTo(Lnet/minecraft/world/level/Level;Lnet/minecraft/core/BlockPos;)Z"),locals = LocalCapture.CAPTURE_FAILSOFT, cancellable = true)
    public void changeLogic(ITurtleAccess turtle, CallbackInfoReturnable<TurtleCommandResult> cir, Direction direction, ServerLevel oldWorld, BlockPos oldPosition, BlockPos newPosition) {
        boolean stable = oldWorld.m_46749_(newPosition);
        if(turtle instanceof ITurtleBrainMixin){
            if(((ITurtleBrainMixin)turtle).lmc$preMove(oldWorld,newPosition)){
                stable = true;
            }
        }
        if(!stable){
            cir.setReturnValue(TurtleCommandResult.failure("Cannot enter unloaded area"));
        }
    }
    //?}

    /**
     * @author Drathonix
     * @reason If anyone else touches the original method, I will be very unhappy.
     */
    @Overwrite
    private static TurtleCommandResult canEnter(TurtlePlayer turtlePlayer,
                                                //? if >1.19.2 {
                                                ServerLevel world,
                                                //?} else {
                                                /*Level world,
                                                *///?}
                                                BlockPos position) {
        if (world.m_151570_(position)) {
            return TurtleCommandResult.failure(position.m_123342_() < 0 ? "Too low to move" : "Too high to move");
        }
        if (!world.m_46739_(position)) return TurtleCommandResult.failure("Cannot leave the world");

        // Check spawn protection
        //? if <=1.19.2 {
        /*if( ComputerCraft.turtlesObeyBlockProtection && !TurtlePermissions.isBlockEnterable( world, position, turtlePlayer )) {
        *///?} else {
        if (turtlePlayer.isBlockProtected(world, position)) {
        //?}
            return TurtleCommandResult.failure("Cannot enter protected area");
        }

        if (!world.m_6857_().m_61937_(position)) {
            return TurtleCommandResult.failure("Cannot pass the world border");
        }

        return TurtleCommandResult.success();
    }
}
//?}

==> loadmychunks-1.2.1%2B1.18.2%2Bforge-sources.jar: com/drathonix/loadmychunks/common/mixin/cct/MixinTurtleMoveCommand.java <==
package com.drathonix.loadmychunks.common.mixin.cct;

//? if !computercraft {
/*import com.drathonix.loadmychunks.common.LoadMyChunks;
import org.spongepowered.asm.mixin.Mixin;

@Mixin(LoadMyChunks.class)
public class MixinTurtleMoveCommand {

}
*///?} else {
import com.drathonix.loadmychunks.common.integ.cct.bridge.ITurtleBrainMixin;
import com.drathonix.loadmychunks.common.integ.cct.turtle.TurtleChunkLoaderPeripheral;
import com.drathonix.loadmychunks.common.system.ChunkDataManager;
import dan200.computercraft.api.peripheral.IPeripheral;
import dan200.computercraft.api.turtle.ITurtleAccess;
import dan200.computercraft.api.turtle.TurtleCommandResult;
import dan200.computercraft.api.turtle.TurtleSide;
//? if <=1.19.2 {
import dan200.computercraft.shared.TurtlePermissions;
import dan200.computercraft.ComputerCraft;
//?}
import dan200.computercraft.shared.turtle.core.TurtleBrain;
import dan200.computercraft.shared.turtle.core.TurtleMoveCommand;
import dan200.computercraft.shared.turtle.core.TurtlePlayer;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.Level;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import org.spongepowered.asm.mixin.injection.callback.LocalCapture;

@SuppressWarnings("all")
@Mixin(value = TurtleMoveCommand.class
//? if forge && <=1.20.1
, remap=false
)
public class MixinTurtleMoveCommand {

    //? if <=1.19.2 {
    @Redirect(method = "execute",at = @At(value = "INVOKE", target = "Ldan200/computercraft/api/turtle/ITurtleAccess;teleportTo(Lnet/minecraft/world/level/Level;Lnet/minecraft/core/BlockPos;)Z")
            //? if forge && <=1.16.5
            /*,remap = true*/
    )
    public boolean checkCanTP(ITurtleAccess turtle, Level oldWorld, BlockPos newPosition){
        boolean stable = oldWorld.m_46749_(newPosition);
        if(oldWorld instanceof ServerLevel) {
            stable = stable || ((ITurtleBrainMixin)turtle).lmc$preMove((ServerLevel)oldWorld,newPosition);
        }
        if(stable){
            return turtle.teleportTo(oldWorld,newPosition);
        }
        return false;
    }
    //?} else {
    /*@Inject(method = "execute",at = @At(value = "INVOKE",target = "Ldan200/computercraft/api/turtle/ITurtleAccess;teleportTo(Lnet/minecraft/world/level/Level;Lnet/minecraft/core/BlockPos;)Z"),locals = LocalCapture.CAPTURE_FAILSOFT, cancellable = true)
    public void changeLogic(ITurtleAccess turtle, CallbackInfoReturnable<TurtleCommandResult> cir, Direction direction, ServerLevel oldWorld, BlockPos oldPosition, BlockPos newPosition) {
        boolean stable = oldWorld.isLoaded(newPosition);
        if(turtle instanceof ITurtleBrainMixin){
            if(((ITurtleBrainMixin)turtle).lmc$preMove(oldWorld,newPosition)){
                stable = true;
            }
        }
        if(!stable){
            cir.setReturnValue(TurtleCommandResult.failure("Cannot enter unloaded area"));
        }
    }
    *///?}

    /**
     * @author Drathonix
     * @reason If anyone else touches the original method, I will be very unhappy.
     */
    @Overwrite
    private static TurtleCommandResult canEnter(TurtlePlayer turtlePlayer,
                                                //? if >1.19.2 {
                                                /*ServerLevel world,
                                                *///?} else {
                                                Level world,
                                                //?}
                                                BlockPos position) {
        if (world.m_151570_(position)) {
            return TurtleCommandResult.failure(position.m_123342_() < 0 ? "Too low to move" : "Too high to move");
        }
        if (!world.m_46739_(position)) return TurtleCommandResult.failure("Cannot leave the world");

        // Check spawn protection
        //? if <=1.19.2 {
        if( ComputerCraft.turtlesObeyBlockProtection && !TurtlePermissions.isBlockEnterable( world, position, turtlePlayer )) {
        //?} else {
        /*if (turtlePlayer.isBlockProtected(world, position)) {
        *///?}
            return TurtleCommandResult.failure("Cannot enter protected area");
        }

        if (!world.m_6857_().m_61937_(position)) {
            return TurtleCommandResult.failure("Cannot pass the world border");
        }

        return TurtleCommandResult.success();
    }
}
//?}
